@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  *
  *  BlueZ - Bluetooth protocol stack for Linux
@@ -5,41 +6,31 @@
  *  Copyright (C) 2012-2013  Tieto Poland
  *
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#define _GNU_SOURCE
+#include <stdlib.h>
 #include <errno.h>
-#include <gdbus/gdbus.h>
 
-#include <bluetooth/bluetooth.h>
-#include <bluetooth/hci.h>
-#include <bluetooth/sdp.h>
+#include "lib/bluetooth.h"
+#include "lib/hci.h"
+#include "lib/sdp.h"
 
-#include "plugin.h"
-#include "log.h"
-#include "dbus-common.h"
-#include "adapter.h"
-#include "device.h"
-#include "eir.h"
-#include "agent.h"
-#include "hcid.h"
+#include "gdbus/gdbus.h"
+
+#include "src/plugin.h"
+#include "src/log.h"
+#include "src/dbus-common.h"
+#include "src/adapter.h"
+#include "src/device.h"
+#include "src/eir.h"
+#include "src/agent.h"
+#include "src/btd.h"
+#include "src/shared/util.h"
 
 #define NEARD_NAME "org.neard"
 #define NEARD_PATH "/"
@@ -77,11 +68,11 @@ struct oob_params {
 
 static void free_oob_params(struct oob_params *params)
 {
-	g_slist_free_full(params->services, g_free);
+	g_slist_free_full(params->services, free);
 	g_free(params->name);
 	g_free(params->hash);
 	g_free(params->randomizer);
-	g_free(params->pin);
+	free(params->pin);
 }
 
 static DBusMessage *error_reply(DBusMessage *msg, int error)
@@ -158,7 +149,7 @@ static void register_agent(bool append_carrier)
 		dbus_message_append_args(message, DBUS_TYPE_STRING, &carrier,
 							DBUS_TYPE_INVALID);
 
-	if (!dbus_connection_send_with_reply(btd_get_dbus_connection(),
+	if (!g_dbus_send_message_with_reply(btd_get_dbus_connection(),
 							message, &call, -1)) {
 		dbus_message_unref(message);
 		error("D-Bus send failed");
@@ -227,12 +218,12 @@ static DBusMessage *create_request_oob_reply(struct btd_adapter *adapter,
 	uint8_t *peir = eir;
 	int len;
 
-	len = eir_create_oob(adapter_get_address(adapter),
+	len = eir_create_oob(btd_adapter_get_address(adapter),
 				btd_adapter_get_name(adapter),
 				btd_adapter_get_class(adapter), hash,
-				randomizer, main_opts.did_vendor,
-				main_opts.did_product, main_opts.did_version,
-				main_opts.did_source,
+				randomizer, btd_opts.did_vendor,
+				btd_opts.did_product, btd_opts.did_version,
+				btd_opts.did_source,
 				btd_adapter_get_services(adapter), eir);
 
 	reply = dbus_message_new_method_return(msg);
@@ -326,7 +317,7 @@ static int check_device(struct btd_device *device)
 		return -ENOENT;
 
 	/* If already paired */
-	if (device_is_paired(device)) {
+	if (device_is_paired(device, BDADDR_BREDR)) {
 		DBG("already paired");
 		return -EALREADY;
 	}
@@ -417,10 +408,10 @@ static int process_nokia_long (void *data, size_t size, uint8_t marker,
 		remote->name = g_strndup((char *)n->name, n->name_len);
 
 	if (marker == 0x01) {
-		remote->pin = g_memdup(n->authentication, 4);
+		remote->pin = util_memdup(n->authentication, 4);
 		remote->pin_len = 4;
 	} else if (marker == 0x02) {
-		remote->pin = g_memdup(n->authentication, 16);
+		remote->pin = util_memdup(n->authentication, 16);
 		remote->pin_len = 16;
 	}
 
@@ -449,7 +440,7 @@ static int process_nokia_short (void *data, size_t size,
 	if (n->name_len > 0)
 		remote->name = g_strndup((char *)n->name, n->name_len);
 
-	remote->pin = g_memdup(n->authentication, 4);
+	remote->pin = util_memdup(n->authentication, 4);
 	remote->pin_len = 4;
 
 	return 0;
@@ -584,14 +575,12 @@ static int process_message(DBusMessage *msg, struct oob_params *remote)
 			if (process_nokia_com_bt(data, size, remote))
 				goto error;
 		} else if (strcasecmp(key, "State") == 0) {
-			DBusMessageIter array;
 			const char *state;
 
 			if (dbus_message_iter_get_arg_type(&value) !=
 					DBUS_TYPE_STRING)
 				goto error;
 
-			dbus_message_iter_recurse(&value, &array);
 			dbus_message_iter_get_basic(&value, &state);
 
 			remote->power_state = process_state(state);
@@ -640,10 +629,11 @@ static void store_params(struct btd_adapter *adapter, struct btd_device *device,
 
 	if (params->name) {
 		device_store_cached_name(device, params->name);
-		device_set_name(device, params->name);
+		btd_device_device_set_name(device, params->name);
 	}
 
-	/* TODO handle UUIDs? */
+	if (params->services)
+		device_add_eir_uuids(device, params->services);
 
 	if (params->hash) {
 		btd_adapter_add_remote_oob_data(adapter, &params->address,
@@ -701,7 +691,8 @@ static DBusMessage *push_oob(DBusConnection *conn, DBusMessage *msg, void *data)
 		return error_reply(msg, EINVAL);
 	}
 
-	device = adapter_get_device(adapter, &remote.address, BDADDR_BREDR);
+	device = btd_adapter_get_device(adapter, &remote.address,
+								BDADDR_BREDR);
 
 	err = check_device(device);
 	if (err < 0) {
@@ -766,41 +757,39 @@ static DBusMessage *request_oob(DBusConnection *conn, DBusMessage *msg,
 	if (err < 0)
 		return error_reply(msg, -err);
 
-	if (bacmp(&remote.address, BDADDR_ANY) == 0)
-		goto read_local;
+	if (bacmp(&remote.address, BDADDR_ANY) == 0) {
+		if (btd_adapter_get_powered(adapter))
+			goto read_local;
 
-	device = adapter_get_device(adapter, &remote.address, BDADDR_BREDR);
-
-	err = check_device(device);
-	if (err < 0) {
-		free_oob_params(&remote);
-
-		if (err == -EALREADY)
-			return create_request_oob_reply(adapter, NULL, NULL,
-									msg);
-
-		return error_reply(msg, -err);
+		goto done;
 	}
 
-	if (!btd_adapter_get_pairable(adapter)) {
-		free_oob_params(&remote);
+	device = btd_adapter_get_device(adapter, &remote.address, BDADDR_BREDR);
 
-		return error_reply(msg, ENONET);
+	err = check_device(device);
+	if (err < 0)
+		goto done;
+
+	if (!btd_adapter_get_pairable(adapter)) {
+		err = -ENONET;
+		goto done;
 	}
 
 	store_params(adapter, device, &remote);
 
-	if (!remote.hash || !btd_adapter_get_powered(adapter)) {
-		free_oob_params(&remote);
-		return create_request_oob_reply(adapter, NULL, NULL, msg);
-	}
+	if (remote.hash && btd_adapter_get_powered(adapter))
+		goto read_local;
+done:
+	free_oob_params(&remote);
+
+	if (err < 0 && err != -EALREADY)
+		return error_reply(msg, -err);
+
+	return create_request_oob_reply(adapter, NULL, NULL, msg);
 
 read_local:
 	free_oob_params(&remote);
 
-	if (!btd_adapter_get_powered(adapter))
-		return create_request_oob_reply(adapter, NULL, NULL, msg);
-
 	err = btd_adapter_read_local_oob_data(adapter);
 	if (err < 0)
 		return error_reply(msg, -err);
