@@ -3400,10 +3400,10 @@ static struct {
 	{ "clock",    cmd_clock,   "Read local or remote clock"           },
 	{ "lescan",   cmd_lescan,  "Start LE scan"                        },
 	{ "leinfo",   cmd_leinfo,  "Get LE remote information"            },
-	{ "lealadd",  cmd_lealadd, "Add device to LE White List"          },
-	{ "lealrm",   cmd_lealrm,  "Remove device from LE White List"     },
-	{ "lealsz",   cmd_lealsz,  "Read size of LE White List"           },
-	{ "lealclr",  cmd_lealclr, "Clear LE White List"                  },
+	{ "lealadd",  cmd_lealadd, "Add device to LE Accept List"         },
+	{ "lealrm",   cmd_lealrm,  "Remove device from LE Accept List"    },
+	{ "lealsz",   cmd_lealsz,  "Read size of LE Accept List"          },
+	{ "lealclr",  cmd_lealclr, "Clear LE Accept List"                 },
 	{ "lewladd",  cmd_lealadd, "Deprecated. Use lealadd instead."     },
 	{ "lewlrm",   cmd_lealrm,  "Deprecated. Use lealrm instead."      },
 	{ "lewlsz",   cmd_lealsz,  "Deprecated. Use lealsz instead."      },
